/**
 * <copyright>
 *
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *   IBM - Initial API and implementation
 *
 * </copyright>
 *
 * $Id$
 */

package org.eclipse.jet;


/**
 * A base class for extending the JET2 context.
 *
 */
public abstract class AbstractContextExtender
{

  private final JET2Context baseContext;

  /**
   * 
   */
  protected AbstractContextExtender(JET2Context context)
  {
    super();
    this.baseContext = context;
    if (!context.hasContextExtender(this.getClass()))
    {
      context.registerContextExtender(this.getClass(), createExtendedData(context));
    }
  }

  /**
   * Called by the AbstractContextExtender constructor if the extender's data
   * has not yet been created in the context.
   * @param context the context in which the data will be created.
   * @return the extension data object.
   */
  protected abstract Object createExtendedData(JET2Context context);

  protected Object getExtendedData()
  {
    return baseContext.getContextExtenderData(this.getClass());
  }

  /**
   * Return the JET2Context that this extender instance is extending.
   * @return the hosting context.
   */
  public final JET2Context getContext()
  {
    return baseContext;
  }
}
