/**
 * <copyright>
 *
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *   IBM - Initial API and implementation
 *
 * </copyright>
 *
 * $Id$
 */
package org.eclipse.jet.internal.compiler;

import org.eclipse.emf.codegen.jet.JETException;
import org.eclipse.jet.compiler.Problem;
import org.eclipse.jet.internal.parser.JETCoreElement;
import org.eclipse.jet.internal.parser.JETMark;
import org.eclipse.jet.internal.parser.JETParseEventListener;
import org.eclipse.jet.internal.parser.JETParseEventListener2;
import org.eclipse.jet.internal.parser.JETParser;
import org.eclipse.jet.internal.parser.JETReader;
import org.eclipse.jet.internal.parser.ParseProblemSeverity;

/**
 * A JETCoreElement that delegates its accept the the passed core element, but records any JETException
 * as errors on a JETParseEventListener2.
 */
public class ErrorRedirectingCoreElementDelegate implements JETCoreElement
{

  private final JETCoreElement delegate;

  public ErrorRedirectingCoreElementDelegate(JETCoreElement delegate) {
    this.delegate = delegate;
  }
  
  /* (non-Javadoc)
   * @see org.eclipse.jet.internal.parser.JETCoreElement#accept(org.eclipse.jet.internal.parser.JETParseEventListener, org.eclipse.jet.internal.parser.JETReader, org.eclipse.jet.internal.parser.JETParser)
   */
  public boolean accept(JETParseEventListener listener, JETReader reader, JETParser parser)
  {
    boolean isAccepted = true;
    JETMark start = reader.mark();
    try {
      isAccepted = delegate.accept(listener, reader, parser);
    }
    catch(JETException e) {
      if(listener instanceof JETParseEventListener2)
      {
        JETParseEventListener2 listener2 = (JETParseEventListener2)listener;
        listener2.recordProblem(ParseProblemSeverity.ERROR, Problem.JETException, e.getLocalizedMessage(), new Object[0], 
          start.getCursor(), reader.mark().getCursor(), start.getLine(), start.getCol());
      }
    }
    return isAccepted;
  }

}
