package org.eclipse.jet.internal.compiler.templates;

import org.eclipse.jet.JET2Context;
import org.eclipse.jet.JET2Template;
import org.eclipse.jet.JET2Writer;
import org.eclipse.jet.taglib.RuntimeTagElement;
import org.eclipse.jet.taglib.TagInfo;
import org.eclipse.jet.internal.compiler.*;
import org.eclipse.jet.compiler.*;
import org.eclipse.jet.transform.TransformContextExtender;
import org.eclipse.jet.taglib.TagLibraryReference;
import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;

public class JET2JavaGenerator implements JET2Template {
    public static final String _jetns_f = "org.eclipse.jet.formatTags"; //$NON-NLS-1$
    public static final String _jetns_c = "org.eclipse.jet.controlTags"; //$NON-NLS-1$
    public static final String _jetns_java = "org.eclipse.jet.javaTags"; //$NON-NLS-1$


    private XMLElement[] getAllXMLTags(JET2CompilationUnit cu) {
        final List result = new ArrayList();
    	cu.accept(new DefaultJET2ASTVisitor() {
	    	public void visit(XMLBodyElement element) {
		    	result.add(element);
		    }
		
			public void visit(XMLEmptyElement element) {
				result.add(element);
			}
		});
        return (XMLElement[])result.toArray(new XMLElement[result.size()]);
    }
    
    private JET2Declaration[] getAllJavaDeclarations(JET2CompilationUnit cu) {
        final List result = new ArrayList();
    	cu.accept(new DefaultJET2ASTVisitor() {
	    	public void visit(JET2Declaration declaration) {
		    	result.add(declaration);
		    }
		});
        return (JET2Declaration[])result.toArray(new JET2Declaration[result.size()]);
    }


	public JET2JavaGenerator() {
		super();
	}

	public void generate(final JET2Context context, JET2Writer out) {
        final TagInfo _jettd_c_include_80_4 = new TagInfo("c:include", //$NON-NLS-1$
            80, 4,
            new String[] {
                "template", //$NON-NLS-1$
            },
            new String[] {
                "templates/tagDataDeclarations.jet", //$NON-NLS-1$
            } );

	JET2CompilationUnit cu = (JET2CompilationUnit)context.getVariable("cu"); //$NON-NLS-1$
	
	final JET2Declaration[] allMemberDeclarations = getAllJavaDeclarations(cu);
	final XMLElement[] allXMLElements = getAllXMLTags(cu);
		
	final TagLibraryReference[] tlRefs = cu.getTagLibraryReferences();

	MethodBodyCreator generatedBody = new MethodBodyCreator(TransformContextExtender.getInstance(context).getLoader());
	cu.accept(generatedBody);
        out.write( "package " ); //$NON-NLS-1$
        out.write( cu.getOutputJavaPackage() );
        out.write( ";\r\n\r\nimport org.eclipse.jet.JET2Context;\r\nimport org.eclipse.jet.JET2Template;\r\nimport org.eclipse.jet.JET2Writer;\r\n" ); //$NON-NLS-1$
 if( allXMLElements.length > 0) {         out.write( "import org.eclipse.jet.taglib.RuntimeTagElement;\r\nimport org.eclipse.jet.taglib.TagInfo;\r\n" ); //$NON-NLS-1$
 }  for(Iterator i=cu.getImports().iterator();i.hasNext();) { 
	String importName = (String)i.next();        out.write( "import " ); //$NON-NLS-1$
        out.write(importName);
        out.write( ";\r\n" ); //$NON-NLS-1$
 }         out.write( "\r\npublic class " ); //$NON-NLS-1$
        out.write( cu.getOutputJavaClassName() );
        out.write( " implements JET2Template {\r\n" ); //$NON-NLS-1$
 if( allXMLElements.length > 0) {     for(int i = 0; i < tlRefs.length; i++ ) {         out.write( "    public static final String _jetns_" ); //$NON-NLS-1$
        out.write( tlRefs[i].getPrefix() );
        out.write( " = \"" ); //$NON-NLS-1$
        out.write( tlRefs[i].getTagLibraryId() );
        out.write( "\"; //$NON-NLS-1$\r\n" ); //$NON-NLS-1$
    }  } 	for(int i = 0; i < allMemberDeclarations.length; i++) {         out.write( "\r\n" ); //$NON-NLS-1$
        out.write( allMemberDeclarations[i].getJavaContent() );
        out.write( "\r\n" ); //$NON-NLS-1$
  }         out.write( "\r\n\tpublic " ); //$NON-NLS-1$
        out.write( cu.getOutputJavaClassName() );
        out.write( "() {\r\n\t\tsuper();\r\n\t}\r\n\r\n\tpublic void generate(final JET2Context context, JET2Writer out) {\r\n" ); //$NON-NLS-1$
	for(int i = 0; i < allXMLElements.length; i++) { 
	    context.setVariable("element", allXMLElements[i]); //$NON-NLS-1$
        RuntimeTagElement _jettag_c_include_80_4 = context.getTagFactory().createRuntimeTag(_jetns_c, "include", "c:include", _jettd_c_include_80_4); //$NON-NLS-1$ //$NON-NLS-2$
        _jettag_c_include_80_4.setRuntimeParent(null);
        _jettag_c_include_80_4.setTagInfo(_jettd_c_include_80_4);
        _jettag_c_include_80_4.doStart(context, out);
        _jettag_c_include_80_4.doEnd();
  }         out.write( generatedBody.getBuffer() );
        out.write( "\r\n\t}\r\n\r\n}\r\n" ); //$NON-NLS-1$

	}

}
