/**
 * <copyright>
 *
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *   IBM - Initial API and implementation
 *
 * </copyright>
 *
 * $Id$
 */
package org.eclipse.jet.internal.runtime;


import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.jet.AbstractContextExtender;
import org.eclipse.jet.JET2Context;
import org.eclipse.jet.taglib.TagInfo;
import org.eclipse.jet.transform.TransformContextExtender;


/**
 * A JET2Context extender that may be installed for logging runtime tag activity.
 */
public class RuntimeLoggerContextExtender extends AbstractContextExtender
{

  public static final int ERROR_LEVEL = 5;
  public static final int WARNING_LEVEL = 4;
  public static final int INFO_LEVEL = 3;
  public static final int TRACE_LEVEL = 2;
  public static final int DEBUG_LEVEL = 1;
  

  private static final class ContextData
  {
    private final List listeners = new ArrayList();
  }

  public RuntimeLoggerContextExtender(JET2Context context)
  {
    super(context);
  }

  /* (non-Javadoc)
   * @see org.eclipse.jet.AbstractContextExtender#createExtendedData(org.eclipse.jet.JET2Context)
   */
  protected Object createExtendedData(JET2Context context)
  {
    return new ContextData();
  }

  private ContextData getData()
  {
    return (ContextData)getExtendedData();
  }

  public void addListener(RuntimeTagLogger logger)
  {
    getData().listeners.add(logger);
  }

  public void removeListener(RuntimeTagLogger logger)
  {
    getData().listeners.remove(logger);
  }

  public static void log(JET2Context context, String message, TagInfo td, int level)
  {
    if (context.hasContextExtender(RuntimeLoggerContextExtender.class))
    {
      RuntimeLoggerContextExtender rl = new RuntimeLoggerContextExtender(context);
      String templatePath = null;
      if(context.hasContextExtender(TransformContextExtender.class)) {
        TransformContextExtender tce = new TransformContextExtender(context);
        templatePath = tce.getTemplatePath();
      }
      rl.log(message, td, templatePath, level);
    }
  }

  public void log(String message, TagInfo td, String templatePath, int level)
  {
    for (Iterator i = getData().listeners.iterator(); i.hasNext();)
    {
      RuntimeTagLogger logger = (RuntimeTagLogger)i.next();
      logger.log(message, td, templatePath, level);
    }
  }

  public static void log(JET2Context context, String message, TagInfo tagInfo, String templatePath, int level)
  {
    if (context.hasContextExtender(RuntimeLoggerContextExtender.class))
    {
      RuntimeLoggerContextExtender rl = new RuntimeLoggerContextExtender(context);
      rl.log(message, tagInfo, templatePath, level);
    }
  }
}
