/**
 * <copyright>
 *
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *   IBM - Initial API and implementation
 *
 * </copyright>
 *
 * $Id$
 */
package org.eclipse.jet.internal.runtime.model;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.util.BasicDiagnostic;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.jet.CoreJETException;

/**
 * A wrapper that makes a CoreException seem like a DiagnosticException.
 */
public class CoreExceptionWrapper extends CoreJETException
{

  public CoreExceptionWrapper(CoreException e)
  {
    super(diagnosticFor(e.getStatus()));
  }

  /**
   * 
   */
  private static final long serialVersionUID = 5977946863300207215L;

  /**
   * Return a Diagnostic equivalent to the passed IStatus.
   * @param status an IStatus.
   * @return the equivalent Diagnostic
   */
  public static Diagnostic diagnosticFor(IStatus status)
  {
    BasicDiagnostic bd = new BasicDiagnostic(status.getSeverity(), status.getPlugin(), status.getCode(), status.getMessage(), new Object[] {status.getException()});
    final List childDiagnostics = childDiagnosticsFor(status);
    for (Iterator i = childDiagnostics.iterator(); i.hasNext();)
    {
      Diagnostic diagnostic = (Diagnostic)i.next();
      bd.add(diagnostic);
    }
    return bd;
  }

  /**
   * Create a list of Diagnostic objects equivalent to the child IStatus objects of the passed IStatus.
   * @param status the IStatus containing the child IStatus objects to wrap.
   * @return a possibly empty List of Diagnostic objects.
   */
  private static List childDiagnosticsFor(IStatus status)
  {
    if(!status.isMultiStatus()) {
      return Collections.EMPTY_LIST;
    }
    final IStatus[] childStati = status.getChildren();
    List result = new ArrayList(childStati.length);
    for (int i = 0; i < childStati.length; i++)
    {
      result.add(diagnosticFor(childStati[i]));
    }
    return result;
  }
}
