/**
 * <copyright>
 *
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *   IBM - Initial API and implementation
 *
 * </copyright>
 *
 * $Id$
 */
package org.eclipse.jet.internal.runtime.model;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import org.eclipse.jet.compiler.JET2CompilationUnit;
import org.eclipse.jet.runtime.model.IModelLoader;

/**
 * Implement the model loader for JET templates
 */
public class JETTemplateModelLoader implements IModelLoader
{

  /**
   * 
   */
  public JETTemplateModelLoader()
  {
    super();
  }

  /* (non-Javadoc)
   * @see org.eclipse.jet.runtime.model.IModelLoader#load(java.net.URL)
   */
  public Object load(URL modelUrl) throws IOException
  {
    return load(modelUrl, "jet"); //$NON-NLS-1$
  }

  /* (non-Javadoc)
   * @see org.eclipse.jet.runtime.model.IModelLoader#load(java.net.URL, java.lang.String)
   */
  public Object load(URL modelUrl, String kind) throws IOException
  {
    JET2CompilationUnit cu = new JET2CompilationUnit();
    final InputStream modelStream = modelUrl.openStream();
    try {
      cu.parse(modelStream, "UTF-8"); //$NON-NLS-1$
    } finally {
      if(modelStream != null) {
          try
          {
            modelStream.close();
          }
          catch (IOException e)
          {
            // nothing we can do, and we should never throw exceptions from finally clauses. 
          }
      }
    }
    return cu;
  }

  /* (non-Javadoc)
   * @see org.eclipse.jet.runtime.model.IModelLoader#loadFromString(java.lang.String, java.lang.String)
   */
  public Object loadFromString(String serializedModel, String kind) throws IOException
  {
    JET2CompilationUnit cu = new JET2CompilationUnit();
    cu.parse(serializedModel);
    return cu;
  }

  /* (non-Javadoc)
   * @see org.eclipse.jet.runtime.model.IModelLoader#canLoad(java.lang.String)
   */
  public boolean canLoad(String kind)
  {
    throw new UnsupportedOperationException();
  }

}
