/**
 * <copyright>
 *
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *   IBM - Initial API and implementation
 *
 * </copyright>
 *
 * $Id$
 */

package org.eclipse.jet.internal.ui.prefs;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.resources.ProjectScope;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.preferences.InstanceScope;
import org.eclipse.jet.JET2Platform;
import org.eclipse.jet.compiler.JET2Compiler;
import org.eclipse.jet.internal.JETPreferences;
import org.eclipse.jet.internal.ui.l10n.Messages;
import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.IWorkbenchPropertyPage;
import org.eclipse.ui.preferences.ScopedPreferenceStore;

/**
 * @author pelder
 *
 */
public class CompilePreferencePage extends FieldEditorPreferencePage implements
		IWorkbenchPreferencePage, IWorkbenchPropertyPage {

	private IProject element;
	private StringFieldEditor packageFieldEditor;
	private ExtensionListEditor extFieldEditor;
	private StringFieldEditor srcFolderFieldEditor;
	private BooleanFieldEditor derivedJavaFieldEditor;
	private BooleanFieldEditor projectSpecificSettingsEditor = null;

	/**
	 * @param style
	 */
	public CompilePreferencePage() {
		super(GRID);
		setDescription(Messages.CompilePreferencePage_Description);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.FieldEditorPreferencePage#createFieldEditors()
	 */
	protected void createFieldEditors() {
		
		if(element == null) {
			setPreferenceStore(new ScopedPreferenceStore(new InstanceScope(), JET2Platform.PLUGIN_ID));
		} else {
			setPreferenceStore(new ScopedPreferenceStore(new ProjectScope(element), JET2Platform.PLUGIN_ID));
			projectSpecificSettingsEditor = new BooleanFieldEditor(JETPreferences.PROJECT_SPECIFIC_SETTINGS, 
											Messages.CompilePreferencePage_EnableProjectSettings, getFieldEditorParent());
			addField(projectSpecificSettingsEditor);
		}

		
		packageFieldEditor = new StringFieldEditor(JET2Compiler.OPTION_COMPILED_TEMPLATE_PACKAGE, 
								Messages.CompilePreferencePage_JavaPackage, getFieldEditorParent());
		addField(packageFieldEditor);
		
		extFieldEditor = new ExtensionListEditor(JET2Compiler.OPTION_TEMPLATE_EXT, 
								Messages.CompilePreferencePage_FileExtensions, getFieldEditorParent());
		addField(extFieldEditor);

		srcFolderFieldEditor = new StringFieldEditor(JET2Compiler.OPTION_COMPILED_TEMPLATE_SRC_DIR, 
								Messages.CompilePreferencePage_SourceFolder, getFieldEditorParent());
		addField(srcFolderFieldEditor);
		
		derivedJavaFieldEditor = new BooleanFieldEditor(JET2Compiler.OPTION_SET_JAVA_FILES_AS_DERIVED, 
								Messages.CompilePreferencePage_DeriveJavaFiles, getFieldEditorParent());
		addField(derivedJavaFieldEditor);

	}

	/**
	 * 
	 */
	private void updateControlEnablement() {
		boolean enabled =  projectSpecificSettingsEditor.getBooleanValue();
		packageFieldEditor.setEnabled(enabled, getFieldEditorParent());
		extFieldEditor.setEnabled(enabled, getFieldEditorParent());
		srcFolderFieldEditor.setEnabled(enabled, getFieldEditorParent());
		derivedJavaFieldEditor.setEnabled(enabled, getFieldEditorParent());
		if(!enabled) {
			packageFieldEditor.loadDefault();
			extFieldEditor.loadDefault();
			srcFolderFieldEditor.loadDefault();
			derivedJavaFieldEditor.loadDefault();
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}

	public IAdaptable getElement() {
		return element;
	}

	public void setElement(IAdaptable element) {
		this.element = (IProject)element;
	}

	protected void initialize() {
		super.initialize();
		if(projectSpecificSettingsEditor != null)
		{
			updateControlEnablement();
		}
	}
	
	public void propertyChange(PropertyChangeEvent event) {
		super.propertyChange(event);
		if(event.getSource() == projectSpecificSettingsEditor && FieldEditor.VALUE.equals(event.getProperty())) {
			updateControlEnablement();
		}
	}
	
	protected void performDefaults() {
		super.performDefaults();
		if (projectSpecificSettingsEditor != null) {
			updateControlEnablement();
		}

	}
	
	public boolean performOk() {
		final boolean ok = super.performOk();
		if(ok) {
			try {
				if(element != null)
				{
					element.build(IncrementalProjectBuilder.CLEAN_BUILD, null);
				}
				else
				{
					ResourcesPlugin.getWorkspace().build(IncrementalProjectBuilder.CLEAN_BUILD, null);
				}
			} catch (CoreException e) {
				// build failed, we don't care
			}
		}
		return ok;
	}
}
