/**
 * <copyright>
 *
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *   IBM - Initial API and implementation
 *
 * </copyright>
 *
 * $Id$
 */
package org.eclipse.jet.internal.xpath.ast;


import java.util.Iterator;

import org.eclipse.jet.internal.xpath.NodeSetImpl;
import org.eclipse.jet.internal.xpath.functions.BooleanFunction;
import org.eclipse.jet.xpath.Context;
import org.eclipse.jet.xpath.NodeSet;


/**
 * Represent an XPath predicate expression
 *
 */
public class Predicate extends NodeSetExpr
{

  private final ExprNode predicateExpr;

  private final NodeSetExpr leftNodeSet;

  /**
   * 
   */
  public Predicate(NodeSetExpr leftNodeSet, ExprNode predicateExpr)
  {
    super();
    this.leftNodeSet = leftNodeSet;
    this.predicateExpr = predicateExpr;
  }

  private NodeSet filter(NodeSet nodeSet, Context context)
  {

    int contextPosition = 1;
    final int contextSize = nodeSet.size();
    NodeSet result = new NodeSetImpl(contextSize);
    for (Iterator i = nodeSet.iterator(); i.hasNext(); contextPosition++)
    {
      Object node = (Object)i.next();
      Context subContext = context.newSubContext(node, contextPosition, contextSize);
      Object exprVal = predicateExpr.evalAsObject(subContext);
      if (exprVal instanceof Number)
      {
        if (((Number)exprVal).doubleValue() == contextPosition)
        {
          result.add(node);
        }
      }
      else if (BooleanFunction.evaluate(exprVal))
      {
        result.add(node);
      }

    }

    return result;

  }

  public NodeSet evalAsNodeSet(Context context)
  {
    NodeSet nodeSet = leftNodeSet.evalAsNodeSet(context);
    return filter(nodeSet, context);
  }

  public String toString()
  {
    return leftNodeSet.toString() + "[" + predicateExpr + "]"; //$NON-NLS-1$//$NON-NLS-2$
  }

}
